#! /usr/bin/perl
# create_user.pl - Created by James A. Pattie, (james@pcxperience.com)
# This script provides the user with a list of available companies and after you select one, it
# then prompts for the info to create a user in the company.

# Copyright (c) 2000-2003 Xperience, Inc. (http://www.pcxperience.com/)
# All rights reserved.  This program is free software; you can redistribute it
# and/or modify it under the same terms as Perl itself.

use strict;
use Portal::Language;
use Portal::Auth;
use Portal::Application;
use Portal::Data::Config;
use Portal::Objects::UserPreferenceObject;
use Portal::Methods;

# instantiate with language 'en' - English
my $langObj = Portal::Language->new(lang => 'en');
if ($langObj->didErrorOccur)
{
  die "Error:  Instantiating the Language Object failed!\n" . $langObj->errorMessage;
}

my $configObj = undef;
eval { $configObj = Portal::Data::Config->new(langObj => $langObj); };
if ($@)
{
  die "Error instantiating Portal::Data::Config->new()!  Error = $@\n";
}
if ($configObj->didErrorOccur)
{
  die $configObj->errorMessage;
}

my $methods = Portal::Methods->new(langObj => $langObj);
if ($methods->didErrorOccur)
{
  myDie(error => $methods->errorMessage, configObj => $configObj);
}

my $portalDB = $methods->portalDBSetup(type => "portal", configObj => $configObj);
if ($methods->didErrorOccur)
{
  myDie(error => $methods->errorMessage, configObj => $configObj);
}
if ($portalDB->didErrorOccur)
{
  myDie(error => $portalDB->errorMessage, configObj => $configObj);
}

my $authObj = Portal::Auth->new(portalDB => $portalDB, langObj => $langObj);
if ($authObj->didErrorOccur)
{
  die $authObj->errorMessage;
}

my $applicationObj = Portal::Application->new(portalDB => $portalDB, langObj => $langObj);
if ($applicationObj->didErrorOccur)
{
  die $applicationObj->errorMessage;
}

# get a list of all companies in the Portal.
my @Companies = $authObj->getListOfCompanies;
if ($authObj->didErrorOccur)
{
  die $authObj->errorMessage;
}

BEGINAGAIN:

# display the list to the user
my $index = -1;
while ($index < 0 || $index > (scalar @Companies - 1))
{
  print "\nSelect Company to work with:\n";
  for (my $i=0; $i < scalar @Companies; $i++)
  {
    print "$i - " . $Companies[$i]->name . "\n";
  }
  print "\nSelect Company (0-" . (scalar @Companies - 1) . "): ";
  $index = <STDIN>;
  chomp $index;
}

my $companyObj = $Companies[$index];
my %userAttributes = ();
my %userPreferences = ();

# prompt for the User Info.
USER_INFO:
print "\nEnter Info for New User Account in Company '$companyObj->{name}':\n\n";

$userAttributes{companyId} = $companyObj->id;
$userAttributes{uname} = getInput(prompt => "User Name", value => "");
$userAttributes{fname} = getInput(prompt => "First Name", value => "");
$userAttributes{mname} = getInput(prompt => "Middle Name", value => "", mask => ".*");
$userAttributes{lname} = getInput(prompt => "Last Name", value => "");
$userAttributes{email} = getInput(prompt => "E-mail", value => "", mask => ".+\\@[\\w-]+\\.\\D+");
PASSWORD:
  my $pass1 = getInput(prompt => "Password", value => "", mask => ".{6,}");
  my $pass2 = getInput(prompt => "Re-Enter Password", value => "", mask => ".{6,}");
  if ($pass1 ne $pass2)
  {
    print "Passwords do not equal!  Retry.\n";
    goto PASSWORD;
  }
  # unescape any @ and %'s.
  $pass1 =~ s/\\@/@/g;
  $pass1 =~ s/\\%/%/g;
$userAttributes{password} = $pass1;
$userAttributes{passCrypted} = 0;  # indicate the password needs to be crypted.
$userAttributes{empId} = getInput(prompt => "Employee ID", value => "", mask => ".*");
$userAttributes{ssn} = getInput(prompt => "SSN", value => "999999999", mask => "\\d{9}");
$userAttributes{language} = getInput(prompt => "Language", value => "en", mask => "[a-z]{2}");
$userAttributes{timeFormat} = getInput(prompt => "Time Format (12=AM/PM, 24=Military Time)", value => "12", mask => "(12|24)");
$userAttributes{tz} = getInput(prompt => "Time Zone", value => "CST", mask => "[A-Z]{3}");
$userAttributes{comment} = getInput(prompt => "Comment", value => "Portal User.", mask => ".*");
$userAttributes{active} = getInput(prompt => "Active", value => "1", mask => "(1|0)");
ADMINISTRATOR:
$userAttributes{admin} = getInput(prompt => "Administrator", value => "0", mask => "(1|0)");
$userAttributes{sysadmin} = getInput(prompt => "System Administrator", value => "0", mask => "(1|0)");
if ($userAttributes{sysadmin} && $userAttributes{admin} == 0)
{
  print "You can not be a System Administrator without being an Administrator!\n";
  goto ADMINISTRATOR;
}
print "\nUser Preference Settings:\n";
$userPreferences{Desktop}{dynamicContent} = getInput(prompt => "Dynamic Content on Desktop", value => "0", mask => "(1|0)");
$userPreferences{Desktop}{dynamicContentConfig} = getConfigDefault(app => "Portal", module => "Desktop", prefName => "dynamicContent_Config_Default");
$userPreferences{Global}{colorScheme} = getConfigDefault(app => "Portal", module => "Global", prefName => "colorScheme_Default");

# create the user.
my $userObj = Portal::Objects::UserObject->new(%userAttributes, langObj => $langObj);
if ($userObj->didErrorOccur)
{
  print $userObj->errorMessage . "\n";
  goto USER_INFO;
}
if (!$authObj->createUser(userObj => $userObj))
{
  die $authObj->errorMessage;
}
$userObj = $authObj->getUserInfo(uname => $userAttributes{uname});
if ($authObj->didErrorOccur)
{
  die $authObj->errorMessage;
}

print "User $userObj->{uname} ($userObj->{fname} $userObj->{lname}) successfully created!\n\n";
print "Don't forget to assign applications to this user.\n\n";

# create any preferences needed.
foreach my $module (keys %userPreferences)
{
  foreach my $pref (keys %{$userPreferences{$module}})
  {
    my $userPreferenceObj = Portal::Objects::UserPreferenceObject->new(id => $userObj->{id}, app => "Portal", name => "$pref", value => $userPreferences{$module}{$pref}, module => $module, langObj => $langObj);
    if ($userPreferenceObj->didErrorOccur)
    {
      die $userPreferenceObj->errorMessage;
    }
    my $result = $authObj->createUserPreference(userPreferenceObj => $userPreferenceObj);
    if ($authObj->didErrorOccur)
    {
      die $authObj->errorMessage;
    }
    if ($result == -1)
    {
      die "Somehow user $userObj->{uname} has already set preference: " . $userPreferenceObj->print . "\n";
    }
    elsif ($result == -2)
    {
      die "App = 'Portal' does not exist!\nThis error should never happen!\n";
    }
    else
    {
      print "Successfully made preference: " . $userPreferenceObj->print . "\n";
    }
  }
}

my $again = "";
while (length $again == 0)
{
  print "Create another user (Y/n)? ";
  $again = <STDIN>;
  chomp $again;
  if (length $again == 0)
  {
    $again = "y";
  }
}
if ($again =~ /^y$/i)
{
  goto BEGINAGAIN;
}

# getInput - prompt, mask, and value
sub getInput
{
  my %args = (prompt => "", mask => ".*", value => "", @_);
  my $prompt = $args{prompt};
  my $mask = $args{mask};
  my $value = $args{value};

  my $input = "";
  my $done = 0;
  while (!$done)
  {
    print "\n$prompt [$value]: ";
    $input = <STDIN>;
    chomp $input;
    if (length $input == 0)
    {
      $input = $value;
    }
    if ($input !~ /^$mask$/)
    {
      print "'$input' is invalid!\n";
      $input = "";
    }
    else
    {
      $done = 1;  # This will let me out even if I'm entering an empty string, as long as that is valid.
    }
  }
  $input =~ s/@/\\@/g;  # make sure that any @'s are escaped.
  $input =~ s/%/\\%/g;  # make sure that any %'s are escaped.
  return $input;
}

# scalar getConfigDefault(app, module, prefName)
sub getConfigDefault
{
  my %args = (app => "", module => "", prefName => "", @_);
  my $callingApp = $args{app};
  my $prefModule = $args{module};
  my $prefName = $args{prefName};

  # get the default for this preference
  my $prefName = $callingApp . "_" . $prefModule . "_" . $prefName;
  my $prefValue = $methods->getConfigValue(name => $prefName, portalDB => $applicationObj->{portalDB});
  if ($methods->didErrorOccur)
  {
    die $methods->errorMessage;
  }
  if (!defined $prefValue)
  {
    die "config entry name = '$prefName' does not exist!\nYou need to upgrade the Database!\n";
  }
  return $prefValue;
}
